package logic

import (
	"context"
	"encoding/json"
	"errors"
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"fusenapi/utils/s3url_to_s3id"
	"gorm.io/gorm"

	"fusenapi/server/product-template-tag/internal/svc"
	"fusenapi/server/product-template-tag/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetProductTemplateTagsLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetProductTemplateTagsLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetProductTemplateTagsLogic {
	return &GetProductTemplateTagsLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *GetProductTemplateTagsLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *GetProductTemplateTagsLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }

func (l *GetProductTemplateTagsLogic) GetProductTemplateTags(req *types.GetProductTemplateTagsReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	if req.Limit <= 0 || req.Limit > 100 {
		req.Limit = 5
	}
	var (
		productTemplateTags []gmodel.FsProductTemplateTags
		err                 error
	)
	//白板用户
	if !userinfo.IsUser() && !userinfo.IsGuest() {
		// 返回固定模板标签列表
		productTemplateTags, err = l.svcCtx.AllModels.FsProductTemplateTags.GetList(l.ctx, 1, req.Limit, 0, 1, "`id` DESC")
		if err != nil {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get template tags")
		}
	} else {
		//获取用户元数据
		userMaterial, err := l.svcCtx.AllModels.FsUserMaterial.FindLatestOne(l.ctx, userinfo.UserId, userinfo.GuestId)
		if err != nil {
			if !errors.Is(err, gorm.ErrRecordNotFound) {
				logx.Error(err)
				return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get ai recommend product template tag list")
			}
			// 返回固定模板标签列表
			productTemplateTags, err = l.svcCtx.AllModels.FsProductTemplateTags.GetList(l.ctx, 1, req.Limit, 0, 1, "`id` DESC")
			if err != nil {
				logx.Error(err)
				return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get template tags")
			}
		} else {
			//元数据是空的
			if userMaterial.Metadata == nil {
				// 返回固定模板标签列表
				productTemplateTags, err = l.svcCtx.AllModels.FsProductTemplateTags.GetList(l.ctx, 1, req.Limit, 0, 1, "`id` DESC")
				if err != nil {
					logx.Error(err)
					return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get template tags")
				}
			} else {
				//解析元数据
				var metaData map[string]interface{}
				if err = json.Unmarshal([]byte(*userMaterial.Metadata), &metaData); err != nil {
					logx.Error(err)
					return resp.SetStatusWithMessage(basic.CodeJsonErr, "failed to parse user metadata")
				}
				var templateTagNameList []string
				b, _ := json.Marshal(metaData["template_tagid"])
				if err = json.Unmarshal(b, &templateTagNameList); err != nil {
					logx.Error(err)
					return resp.SetStatusWithMessage(basic.CodeJsonErr, "invalid format of metadata`s template_tagid")
				}
				productTemplateTags, err = l.svcCtx.AllModels.FsProductTemplateTags.GetListByTagNames(l.ctx, templateTagNameList, req.Limit, 0, 1, "id DESC")
				if err != nil {
					logx.Error(err)
					return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get template tags")
				}
			}
		}
	}
	//资源id集合
	resourceIds := make([]string, 0, 5)
	for _, v := range productTemplateTags {
		resourceIds = append(resourceIds, s3url_to_s3id.GetS3ResourceIdFormUrl(*v.Cover))
	}
	//根据resourceUrls找到对应的元数据
	resourceMetadataList, err := l.svcCtx.AllModels.FsResource.FindAllByResourceIds(l.ctx, resourceIds)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get resource list")
	}
	mapResourceMetadata := make(map[string]map[string]interface{})
	for _, v := range resourceMetadataList {
		var metadata map[string]interface{}
		if v.Metadata != nil {
			_ = json.Unmarshal([]byte(*v.Metadata), &metadata)
		}
		mapResourceMetadata[*v.ResourceUrl] = metadata
	}
	list := make([]types.GetProductTemplateTagsRsp, 0, len(productTemplateTags))
	for _, v := range productTemplateTags {
		list = append(list, types.GetProductTemplateTagsRsp{
			Id:            v.Id,
			TemplateTag:   *v.TemplateTag,
			Cover:         *v.Cover,
			CoverMetadata: mapResourceMetadata[*v.Cover],
		})
	}
	return resp.SetStatusWithMessage(basic.CodeOK, "success", list)
}
